@@ -12,8 +12,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone import auth
 from keystone.auth import plugins as auth_plugins
+from keystone.auth.plugins import base
 from keystone.common import dependency
 from keystone import exception
 from keystone.i18n import _
@@ -23,15 +23,15 @@ METHOD_NAME = 'password'
 
 
 @dependency.requires('identity_api')
-class Password(auth.AuthMethodHandler):
+class Password(base.AuthMethodHandler):
 
-    def authenticate(self, context, auth_payload, auth_context):
+    def authenticate(self, request, auth_payload, auth_context):
         """Try to authenticate against the identity backend."""
         user_info = auth_plugins.UserAuthInfo.create(auth_payload, METHOD_NAME)
 
         try:
             self.identity_api.authenticate(
-                context,
+                request,
                 user_id=user_info.user_id,
                 password=user_info.password)
         except AssertionError:
