@@ -25,21 +25,23 @@ from oslo_log import versionutils
 from oslo_serialization import jsonutils
 import pbr.version
 
-from keystone.common import config
+from keystone.cmd import doctor
 from keystone.common import driver_hints
 from keystone.common import openssl
 from keystone.common import sql
 from keystone.common.sql import migration_helpers
 from keystone.common import utils
+import keystone.conf
+from keystone.credential.providers import fernet as credential_fernet
 from keystone import exception
 from keystone.federation import idp
 from keystone.federation import utils as mapping_engine
-from keystone.i18n import _, _LW, _LI
+from keystone.i18n import _, _LE, _LI, _LW
 from keystone.server import backends
 from keystone import token
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
 
@@ -55,7 +57,7 @@ class BaseApp(object):
 
 
 class BootStrap(BaseApp):
-    """Perform the basic bootstrap process"""
+    """Perform the basic bootstrap process."""
 
     name = "bootstrap"
 
@@ -264,7 +266,7 @@ class BootStrap(BaseApp):
                 role={'name': self.role_name,
                       'id': self.role_id},
             )
-            LOG.info(_LI('Created Role %s'), self.role_name)
+            LOG.info(_LI('Created role %s'), self.role_name)
         except exception.Conflict:
             LOG.info(_LI('Role %s exists, skipping creation.'), self.role_name)
             # NOTE(davechen): There is no backend method to get the role
@@ -300,7 +302,7 @@ class BootStrap(BaseApp):
                 self.catalog_manager.create_region(
                     region_ref={'id': self.region_id}
                 )
-                LOG.info(_LI('Created Region %s'), self.region_id)
+                LOG.info(_LI('Created region %s'), self.region_id)
             except exception.Conflict:
                 LOG.info(_LI('Region %s exists, skipping creation.'),
                          self.region_id)
@@ -369,12 +371,39 @@ class BootStrap(BaseApp):
 
                 self.endpoints[interface] = endpoint_ref['id']
 
+        self.assignment_manager.ensure_default_role()
+
     @classmethod
     def main(cls):
         klass = cls()
         klass.do_bootstrap()
 
 
+class Doctor(BaseApp):
+    """Diagnose common problems with keystone deployments."""
+
+    name = 'doctor'
+
+    @classmethod
+    def add_argument_parser(cls, subparsers):
+        parser = super(Doctor, cls).add_argument_parser(subparsers)
+        return parser
+
+    @staticmethod
+    def main():
+        # Return a non-zero exit code if we detect any symptoms.
+        raise SystemExit(doctor.diagnose())
+
+
+def assert_not_extension(extension):
+    if extension:
+        print(_("All extensions have been moved into keystone core and as "
+                "such its migrations are maintained by the main keystone "
+                "database control. Use the command: keystone-manage "
+                "db_sync"))
+        raise RuntimeError
+
+
 class DbSync(BaseApp):
     """Sync the database."""
 
@@ -390,17 +419,46 @@ class DbSync(BaseApp):
                                   'version. Schema downgrades are not '
                                   'supported.'))
         parser.add_argument('--extension', default=None,
-                            help=('Migrate the database for the specified '
-                                  'extension. If not provided, db_sync will '
-                                  'migrate the common repository.'))
+                            help=('This is a deprecated option to migrate a '
+                                  'specified extension. Since extensions are '
+                                  'now part of the main repository, '
+                                  'specifying db_sync without this option '
+                                  'will cause all extensions to be migrated.'))
+        group = parser.add_mutually_exclusive_group()
+        group.add_argument('--expand', default=False, action='store_true',
+                           help=('Expand the database schema in preparation '
+                                 'for data migration.'))
+        group.add_argument('--migrate', default=False,
+                           action='store_true',
+                           help=('Copy all data that needs to be migrated '
+                                 'within the database ahead of starting the '
+                                 'first keystone node upgraded to the new '
+                                 'release. This command should be run '
+                                 'after the --expand command. Once the '
+                                 '--migrate command has completed, you can '
+                                 'upgrade all your keystone nodes to the new '
+                                 'release and restart them.'))
 
+        group.add_argument('--contract', default=False, action='store_true',
+                           help=('Remove any database tables and columns '
+                                 'that are no longer required. This command '
+                                 'should be run after all keystone nodes are '
+                                 'running the new release.'))
         return parser
 
     @staticmethod
     def main():
-        version = CONF.command.version
-        extension = CONF.command.extension
-        migration_helpers.sync_database_to_version(extension, version)
+        assert_not_extension(CONF.command.extension)
+
+        if CONF.command.expand:
+            migration_helpers.expand_schema()
+        elif CONF.command.migrate:
+            migration_helpers.migrate_data()
+        elif CONF.command.contract:
+            migration_helpers.contract_schema()
+        else:
+            migration_helpers.offline_sync_database_to_version(
+                CONF.command.version)
 
 
 class DbVersion(BaseApp):
@@ -412,15 +470,17 @@ class DbVersion(BaseApp):
     def add_argument_parser(cls, subparsers):
         parser = super(DbVersion, cls).add_argument_parser(subparsers)
         parser.add_argument('--extension', default=None,
-                            help=('Print the migration version of the '
-                                  'database for the specified extension. If '
-                                  'not provided, print it for the common '
+                            help=('This is a deprecated option to print the '
+                                  'version of a specified extension. Since '
+                                  'extensions are now part of the main '
+                                  'repository, the version of an extension is '
+                                  'implicit in the version of the main '
                                   'repository.'))
 
     @staticmethod
     def main():
-        extension = CONF.command.extension
-        migration_helpers.print_db_version(extension)
+        assert_not_extension(CONF.command.extension)
+        migration_helpers.print_db_version()
 
 
 class BasePermissionsSetup(BaseApp):
@@ -495,25 +555,6 @@ class PKISetup(BaseCertificateSetup):
         conf_pki.run()
 
 
-class SSLSetup(BaseCertificateSetup):
-    """Create key pairs and certificates for HTTPS connections.
-
-    This is NOT intended for production use, see Keystone Configuration
-    documentation for details.
-    """
-
-    name = 'ssl_setup'
-
-    @classmethod
-    def main(cls):
-        LOG.warning(_LW('keystone-manage ssl_setup is not recommended for '
-                        'production use.'))
-        keystone_user_id, keystone_group_id = cls.get_user_group()
-        conf_ssl = openssl.ConfigureSSL(keystone_user_id, keystone_group_id,
-                                        rebuild=CONF.command.rebuild)
-        conf_ssl.run()
-
-
 class FernetSetup(BasePermissionsSetup):
     """Setup a key repository for Fernet tokens.
 
@@ -527,12 +568,16 @@ class FernetSetup(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        from keystone.token.providers.fernet import utils as fernet
+        from keystone.common import fernet_utils as utils
+        fernet_utils = utils.FernetUtils(
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys
+        )
 
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        fernet.create_key_directory(keystone_user_id, keystone_group_id)
-        if fernet.validate_key_repository(requires_write=True):
-            fernet.initialize_key_repository(
+        fernet_utils.create_key_directory(keystone_user_id, keystone_group_id)
+        if fernet_utils.validate_key_repository(requires_write=True):
+            fernet_utils.initialize_key_repository(
                 keystone_user_id, keystone_group_id)
 
 
@@ -558,11 +603,170 @@ class FernetRotate(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        from keystone.token.providers.fernet import utils as fernet
+        from keystone.common import fernet_utils as utils
+        fernet_utils = utils.FernetUtils(
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys
+        )
 
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        if fernet.validate_key_repository(requires_write=True):
-            fernet.rotate_keys(keystone_user_id, keystone_group_id)
+        if fernet_utils.validate_key_repository(requires_write=True):
+            fernet_utils.rotate_keys(keystone_user_id, keystone_group_id)
+
+
+class CredentialSetup(BasePermissionsSetup):
+    """Setup a Fernet key repository for credential encryption.
+
+    The purpose of this command is very similar to `keystone-manage
+    fernet_setup` only the keys included in this repository are for encrypting
+    and decrypting credential secrets instead of token payloads. Keys can be
+    rotated using `keystone-manage credential_rotate`.
+    """
+
+    name = 'credential_setup'
+
+    @classmethod
+    def main(cls):
+        from keystone.common import fernet_utils as utils
+        fernet_utils = utils.FernetUtils(
+            CONF.credential.key_repository,
+            credential_fernet.MAX_ACTIVE_KEYS
+        )
+
+        keystone_user_id, keystone_group_id = cls.get_user_group()
+        fernet_utils.create_key_directory(keystone_user_id, keystone_group_id)
+        if fernet_utils.validate_key_repository(requires_write=True):
+            fernet_utils.initialize_key_repository(
+                keystone_user_id,
+                keystone_group_id
+            )
+
+
+class CredentialRotate(BasePermissionsSetup):
+    """Rotate Fernet encryption keys for credential encryption.
+
+    This assumes you have already run `keystone-manage credential_setup`.
+
+    A new primary key is placed into rotation only if all credentials are
+    encrypted with the current primary key. If any credentials are encrypted
+    with a secondary key the rotation will abort. This protects against
+    removing a key that is still required to decrypt credentials. Once a key is
+    removed from the repository, it is impossible to recover the original data
+    without restoring from a backup external to keystone (more on backups
+    below). To make sure all credentials are encrypted with the latest primary
+    key, please see the `keystone-manage credential_migrate` command. Since the
+    maximum number of keys in the credential repository is 3, once all
+    credentials are encrypted with the latest primary key we can safely
+    introduce a new primary key. All credentials will still be decryptable
+    since they are all encrypted with the only secondary key in the repository.
+
+    It is imperitive to understand the importance of backing up keys used to
+    encrypt credentials. In the event keys are overrotated, applying a key
+    repository from backup can help recover otherwise useless credentials.
+    Persisting snapshots of the key repository in secure and encrypted source
+    control, or a dedicated key management system are good examples of
+    encryption key backups.
+
+    The `keystone-manage credential_rotate` and `keystone-manage
+    credential_migrate` commands are intended to be done in sequence. After
+    performing a rotation, a migration must be done before performing another
+    rotation. This ensures we don't over-rotate encryption keys.
+
+    """
+
+    name = 'credential_rotate'
+
+    def __init__(self):
+        drivers = backends.load_backends()
+        self.credential_provider_api = drivers['credential_provider_api']
+        self.credential_api = drivers['credential_api']
+
+    def validate_primary_key(self):
+        crypto, keys = credential_fernet.get_multi_fernet_keys()
+        primary_key_hash = credential_fernet.primary_key_hash(keys)
+
+        credentials = self.credential_api.driver.list_credentials(
+            driver_hints.Hints()
+        )
+        for credential in credentials:
+            if credential['key_hash'] != primary_key_hash:
+                msg = _('Unable to rotate credential keys because not all '
+                        'credentials are encrypted with the primary key. '
+                        'Please make sure all credentials have been encrypted '
+                        'with the primary key using `keystone-manage '
+                        'credential_migrate`.')
+                raise SystemExit(msg)
+
+    @classmethod
+    def main(cls):
+        from keystone.common import fernet_utils as utils
+        fernet_utils = utils.FernetUtils(
+            CONF.credential.key_repository,
+            credential_fernet.MAX_ACTIVE_KEYS
+        )
+
+        keystone_user_id, keystone_group_id = cls.get_user_group()
+        if fernet_utils.validate_key_repository(requires_write=True):
+            klass = cls()
+            klass.validate_primary_key()
+            fernet_utils.rotate_keys(keystone_user_id, keystone_group_id)
+
+
+class CredentialMigrate(BasePermissionsSetup):
+    """Provides the ability to encrypt credentials using a new primary key.
+
+    This assumes that there is already a credential key repository in place and
+    that the database backend has been upgraded to at least the Newton schema.
+    If the credential repository doesn't exist yet, you can use
+    ``keystone-manage credential_setup`` to create one.
+
+    """
+
+    name = 'credential_migrate'
+
+    def __init__(self):
+        drivers = backends.load_backends()
+        self.credential_provider_api = drivers['credential_provider_api']
+        self.credential_api = drivers['credential_api']
+
+    def migrate_credentials(self):
+        crypto, keys = credential_fernet.get_multi_fernet_keys()
+        primary_key_hash = credential_fernet.primary_key_hash(keys)
+
+        # FIXME(lbragstad): We *should* be able to use Hints() to ask only for
+        # credentials that have a key_hash equal to a secondary key hash or
+        # None, but Hints() doesn't seem to honor None values. See
+        # https://bugs.launchpad.net/keystone/+bug/1614154.  As a workaround -
+        # we have to ask for *all* credentials and filter them ourselves.
+        credentials = self.credential_api.driver.list_credentials(
+            driver_hints.Hints()
+        )
+        for credential in credentials:
+            if credential['key_hash'] != primary_key_hash:
+                # If the key_hash isn't None but doesn't match the
+                # primary_key_hash, then we know the credential was encrypted
+                # with a secondary key. Let's decrypt it, and send it through
+                # the update path to re-encrypt it with the new primary key.
+                decrypted_blob = self.credential_provider_api.decrypt(
+                    credential['encrypted_blob']
+                )
+                cred = {'blob': decrypted_blob}
+                self.credential_api.update_credential(
+                    credential['id'],
+                    cred
+                )
+
+    @classmethod
+    def main(cls):
+        # Check to make sure we have a repository that works...
+        from keystone.common import fernet_utils as utils
+        fernet_utils = utils.FernetUtils(
+            CONF.credential.key_repository,
+            credential_fernet.MAX_ACTIVE_KEYS
+        )
+        fernet_utils.validate_key_repository(requires_write=True)
+        klass = cls()
+        klass.migrate_credentials()
 
 
 class TokenFlush(BaseApp):
@@ -573,7 +777,14 @@ class TokenFlush(BaseApp):
     @classmethod
     def main(cls):
         token_manager = token.persistence.PersistenceManager()
-        token_manager.flush_expired_tokens()
+        try:
+            token_manager.flush_expired_tokens()
+        except exception.NotImplemented:
+            # NOTE(ravelar159): Stop NotImplemented from unsupported token
+            # driver when using token_flush and print out warning instead
+            LOG.warning(_LW('Token driver %s does not support token_flush. '
+                            'The token_flush command had no effect.'),
+                        CONF.token.driver)
 
 
 class MappingPurge(BaseApp):
@@ -663,10 +874,10 @@ DOMAIN_CONF_FTAIL = '.conf'
 def _domain_config_finder(conf_dir):
     """Return a generator of all domain config files found in a directory.
 
-    Donmain configs match the filename pattern of
+    Domain configs match the filename pattern of
     'keystone.<domain_name>.conf'.
 
-    :returns: generator yeilding (filename, domain_name) tuples
+    :returns: generator yielding (filename, domain_name) tuples
     """
     LOG.info(_LI('Scanning %r for domain config files'), conf_dir)
     for r, d, f in os.walk(conf_dir):
@@ -710,29 +921,22 @@ class DomainConfigUploadFiles(object):
                 CONF.command.domain_name is None):
             print(_('At least one option must be provided, use either '
                     '--all or --domain-name'))
-            raise ValueError
+            return False
 
         if (CONF.command.all is True and
                 CONF.command.domain_name is not None):
             print(_('The --all option cannot be used with '
                     'the --domain-name option'))
-            raise ValueError
+            return False
 
-    def upload_config_to_database(self, file_name, domain_name):
+        return True
+
+    def _upload_config_to_database(self, file_name, domain_name):
         """Upload a single config file to the database.
 
         :param file_name: the file containing the config options
         :param domain_name: the domain name
-
-        :raises ValueError: the domain does not exist or already has domain
-            specific configurations defined.
-        :raises Exceptions from oslo config: there is an issue with options
-            defined in the config file or its format.
-
-        The caller of this method should catch the errors raised and handle
-        appropriately in order that the best UX experience can be provided for
-        both the case of when a user has asked for a specific config file to
-        be uploaded, as well as all config files in a directory.
+        :returns: a boolean indicating if the upload succeeded
 
         """
         try:
@@ -743,7 +947,7 @@ class DomainConfigUploadFiles(object):
                     'name: %(file)s - ignoring this file.') % {
                         'domain': domain_name,
                         'file': file_name})
-            raise ValueError
+            return False
 
         if self.domain_config_manager.get_config_with_sensitive_info(
                 domain_ref['id']):
@@ -751,7 +955,7 @@ class DomainConfigUploadFiles(object):
                     'defined - ignoring file: %(file)s.') % {
                         'domain': domain_name,
                         'file': file_name})
-            raise ValueError
+            return False
 
         sections = {}
         try:
@@ -765,40 +969,24 @@ class DomainConfigUploadFiles(object):
                     'file: %(file)s.') % {
                         'domain': domain_name,
                         'file': file_name})
-            raise
+            return False
 
-        for group in sections:
-            for option in sections[group]:
-                    sections[group][option] = sections[group][option][0]
-        self.domain_config_manager.create_config(domain_ref['id'], sections)
-
-    def upload_configs_to_database(self, file_name, domain_name):
-        """Upload configs from file and load into database.
-
-        This method will be called repeatedly for all the config files in the
-        config directory. To provide a better UX, we differentiate the error
-        handling in this case (versus when the user has asked for a single
-        config file to be uploaded).
-
-        """
         try:
-            self.upload_config_to_database(file_name, domain_name)
-        except ValueError:  # nosec
-            # We've already given all the info we can in a message, so carry
-            # on to the next one
-            pass
-        except Exception:
-            # Some other error occurred relating to this specific config file
-            # or domain. Since we are trying to upload all the config files,
-            # we'll continue and hide this exception. However, we tell the
-            # user how to get more info about this error by re-running with
-            # just the domain at fault. When we run in single-domain mode we
-            # will NOT hide the exception.
-            print(_('To get a more detailed information on this error, re-run '
-                    'this command for the specific domain, i.e.: '
-                    'keystone-manage domain_config_upload --domain-name %s') %
-                  domain_name)
-            pass
+            for group in sections:
+                for option in sections[group]:
+                        sections[group][option] = sections[group][option][0]
+            self.domain_config_manager.create_config(domain_ref['id'],
+                                                     sections)
+            return True
+        except Exception as e:
+            msg = _LE('Error processing config file for domain: '
+                      '%(domain_name)s, file: %(filename)s, error: %(error)s')
+            LOG.error(msg,
+                      {'domain_name': domain_name,
+                       'filename': file_name,
+                       'error': e},
+                      exc_info=True)
+            return False
 
     def read_domain_configs_from_files(self):
         """Read configs from file(s) and load into database.
@@ -818,12 +1006,15 @@ class DomainConfigUploadFiles(object):
         if domain_name:
             # Request is to upload the configs for just one domain
             fname = DOMAIN_CONF_FHEAD + domain_name + DOMAIN_CONF_FTAIL
-            self.upload_config_to_database(
-                os.path.join(conf_dir, fname), domain_name)
-            return
+            if not self._upload_config_to_database(
+                    os.path.join(conf_dir, fname), domain_name):
+                return False
+            return True
 
         for filename, domain_name in self._domain_config_finder(conf_dir):
-            self.upload_configs_to_database(filename, domain_name)
+            self._upload_config_to_database(filename, domain_name)
+
+        return True
 
     def run(self):
         # First off, let's just check we can talk to the domain database
@@ -836,12 +1027,10 @@ class DomainConfigUploadFiles(object):
                     'is configured correctly.'))
             raise
 
-        try:
-            self.valid_options()
-            self.read_domain_configs_from_files()
-        except ValueError:
-            # We will already have printed out a nice message, so indicate
-            # to caller the non-success error code to be used.
+        if not self.valid_options():
+            return 1
+
+        if not self.read_domain_configs_from_files():
             return 1
 
 
@@ -867,6 +1056,11 @@ class DomainConfigUpload(BaseApp):
 
     @staticmethod
     def main():
+        versionutils.report_deprecated_feature(
+            LOG,
+            _LW("keystone-manage domain_config_upload is deprecated as of "
+                "Newton in favor of setting domain config options via the API "
+                "and may be removed in 'P' release."))
         dcu = DomainConfigUploadFiles()
         status = dcu.run()
         if status is not None:
@@ -981,18 +1175,68 @@ class MappingEngineTester(BaseApp):
                                   "engine."))
 
 
+class MappingPopulate(BaseApp):
+    """Pre-populate entries from domain-specific backends.
+
+    Running this command is not required. It should only be run right after
+    the LDAP was configured, when many new users were added, or when
+    "mapping_purge" is run.
+
+    This command will take a while to run. It is perfectly fine for it to run
+    more than several minutes.
+    """
+
+    name = "mapping_populate"
+
+    @classmethod
+    def load_backends(cls):
+        drivers = backends.load_backends()
+        cls.identity_api = drivers['identity_api']
+        cls.resource_api = drivers['resource_api']
+
+    @classmethod
+    def add_argument_parser(cls, subparsers):
+        parser = super(MappingPopulate, cls).add_argument_parser(
+            subparsers)
+
+        parser.add_argument('--domain-name', default=None, required=True,
+                            help=("Name of the domain configured to use "
+                                  "domain-specific backend"))
+        return parser
+
+    @classmethod
+    def main(cls):
+        """Process entries for id_mapping_api."""
+        cls.load_backends()
+        domain_name = CONF.command.domain_name
+        try:
+            domain_id = cls.resource_api.get_domain_by_name(domain_name)['id']
+        except exception.DomainNotFound:
+            print(_('Invalid domain name or ID: %(domain)s') % {
+                'domain': domain_id})
+            return False
+        # We don't actually need to tackle id_mapping_api in order to get
+        # entries there, because list_users does this anyway. That's why it
+        # will be enough to just make the call below.
+        cls.identity_api.list_users(domain_scope=domain_id)
+
+
 CMDS = [
     BootStrap,
+    CredentialMigrate,
+    CredentialRotate,
+    CredentialSetup,
     DbSync,
     DbVersion,
+    Doctor,
     DomainConfigUpload,
     FernetRotate,
     FernetSetup,
+    MappingPopulate,
     MappingPurge,
     MappingEngineTester,
     PKISetup,
     SamlIdentityProviderMetadata,
-    SSLSetup,
     TokenFlush,
 ]
 
@@ -1011,14 +1255,16 @@ command_opt = cfg.SubCommandOpt('command',
 def main(argv=None, config_files=None):
     CONF.register_cli_opt(command_opt)
 
-    config.configure()
+    keystone.conf.configure()
     sql.initialize()
-    config.set_default_for_default_log_levels()
+    keystone.conf.set_default_for_default_log_levels()
 
     CONF(args=argv[1:],
          project='keystone',
          version=pbr.version.VersionInfo('keystone').version_string(),
          usage='%(prog)s [' + '|'.join([cmd.name for cmd in CMDS]) + ']',
          default_config_files=config_files)
-    config.setup_logging()
+    if not CONF.default_config_files:
+        LOG.warning(_LW('Config file not found, using default configs.'))
+    keystone.conf.setup_logging()
     CONF.command.cmd_class.main()
