@@ -10,12 +10,16 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from oslo_log import log
+from oslo_serialization import msgpackutils
 from oslo_utils import timeutils
-from six.moves import map
 
+from keystone.common import cache
 from keystone.common import utils
 
 
+LOG = log.getLogger(__name__)
+
 # The set of attributes common between the RevokeEvent
 # and the dictionaries created from the token Data.
 _NAMES = ['trust_id',
@@ -109,150 +113,117 @@ class RevokeEvent(object):
             event['OS-TRUST:trust_id'] = self.trust_id
         if self.consumer_id is not None:
             event['OS-OAUTH1:consumer_id'] = self.consumer_id
-        if self.consumer_id is not None:
+        if self.access_token_id is not None:
             event['OS-OAUTH1:access_token_id'] = self.access_token_id
         if self.expires_at is not None:
             event['expires_at'] = utils.isotime(self.expires_at)
         if self.issued_before is not None:
             event['issued_before'] = utils.isotime(self.issued_before,
                                                    subsecond=True)
+        if self.revoked_at is not None:
+            event['revoked_at'] = utils.isotime(self.revoked_at,
+                                                subsecond=True)
         return event
 
-    def key_for_name(self, name):
-        return "%s=%s" % (name, getattr(self, name) or '*')
 
+def is_revoked(events, token_data):
+    """Check if a token matches a revocation event.
 
-def attr_keys(event):
-    return list(map(event.key_for_name, _EVENT_NAMES))
-
-
-class RevokeTree(object):
-    """Fast Revocation Checking Tree Structure
-
-    The Tree is an index to quickly match tokens against events.
-    Each node is a hashtable of key=value combinations from revocation events.
-    The
+    Compare a token against every revocation event. If the token matches an
+    event in the `events` list, the token is revoked. If the token is compared
+    against every item in the list without a match, it is not considered
+    revoked from the `revoke_api`.
 
+    :param events: a list of RevokeEvent instances
+    :param token_data: map based on a flattened view of the token. The required
+                       fields are `expires_at`,`user_id`, `project_id`,
+                       `identity_domain_id`, `assignment_domain_id`,
+                       `trust_id`, `trustor_id`, `trustee_id` `consumer_id` and
+                       `access_token_id`
+    :returns: True if the token matches an existing revocation event, meaning
+              the token is revoked. False is returned if the token does not
+              match any revocation events, meaning the token is considered
+              valid by the revocation API.
     """
+    return any([matches(e, token_data) for e in events])
 
-    def __init__(self, revoke_events=None):
-        self.revoke_map = dict()
-        self.add_events(revoke_events)
 
-    def add_event(self, event):
-        """Updates the tree based on a revocation event.
+def matches(event, token_values):
+    """See if the token matches the revocation event.
 
-        Creates any necessary internal nodes in the tree corresponding to the
-        fields of the revocation event.  The leaf node will always be set to
-        the latest 'issued_before' for events that are otherwise identical.
+    A brute force approach to checking.
+    Compare each attribute from the event with the corresponding
+    value from the token.  If the event does not have a value for
+    the attribute, a match is still possible.  If the event has a
+    value for the attribute, and it does not match the token, no match
+    is possible, so skip the remaining checks.
 
-        :param:  Event to add to the tree
+    :param event: a RevokeEvent instance
+    :param token_values: dictionary with set of values taken from the
+                         token
+    :returns: True if the token matches the revocation event, indicating the
+              token has been revoked
+    """
+    # If any one check does not match, the whole token does
+    # not match the event. The numerous return False indicate
+    # that the token is still valid and short-circuits the
+    # rest of the logic.
 
-        :returns:  the event that was passed in.
-
-        """
-        revoke_map = self.revoke_map
-        for key in attr_keys(event):
-            revoke_map = revoke_map.setdefault(key, {})
-        revoke_map['issued_before'] = max(
-            event.issued_before, revoke_map.get(
-                'issued_before', event.issued_before))
-        return event
-
-    def remove_event(self, event):
-        """Update the tree based on the removal of a Revocation Event
-
-        Removes empty nodes from the tree from the leaf back to the root.
-
-        If multiple events trace the same path, but have different
-        'issued_before' values, only the last is ever stored in the tree.
-        So only an exact match on 'issued_before' ever triggers a removal
-
-        :param: Event to remove from the tree
-
-        """
-        stack = []
-        revoke_map = self.revoke_map
-        for name in _EVENT_NAMES:
-            key = event.key_for_name(name)
-            nxt = revoke_map.get(key)
-            if nxt is None:
-                break
-            stack.append((revoke_map, key, nxt))
-            revoke_map = nxt
-        else:
-            if event.issued_before == revoke_map['issued_before']:
-                revoke_map.pop('issued_before')
-        for parent, key, child in reversed(stack):
-            if not any(child):
-                del parent[key]
-
-    def add_events(self, revoke_events):
-        return list(map(self.add_event, revoke_events or []))
-
-    @staticmethod
-    def _next_level_keys(name, token_data):
-        """Generate keys based on current field name and token data
-
-        Generate all keys to look for in the next iteration of revocation
-        event tree traversal.
-        """
-        yield '*'
-        if name == 'role_id':
-            # Roles are very special since a token has a list of them.
-            # If the revocation event matches any one of them,
-            # revoke the token.
-            for role_id in token_data.get('roles', []):
-                yield role_id
-        else:
-            # For other fields we try to get any branch that concur
-            # with any alternative field in the token.
-            for alt_name in ALTERNATIVES.get(name, [name]):
-                yield token_data[alt_name]
-
-    def _search(self, revoke_map, names, token_data):
-        """Search for revocation event by token_data
-
-        Traverse the revocation events tree looking for event matching token
-        data issued after the token.
-        """
-        if not names:
-            # The last (leaf) level is checked in a special way because we
-            # verify issued_at field differently.
-            try:
-                return revoke_map['issued_before'] >= token_data['issued_at']
-            except KeyError:
-                return False
-
-        name, remaining_names = names[0], names[1:]
-
-        for key in self._next_level_keys(name, token_data):
-            subtree = revoke_map.get('%s=%s' % (name, key))
-            if subtree and self._search(subtree, remaining_names, token_data):
-                return True
-
-        # If we made it out of the loop then no element in revocation tree
-        # corresponds to our token and it is good.
+    # The token has three attributes that can match the user_id.
+    if event.user_id is not None and event.user_id not in (
+            token_values['user_id'],
+            token_values['trustor_id'],
+            token_values['trustee_id'],):
         return False
 
-    def is_revoked(self, token_data):
-        """Check if a token matches the revocation event
+    # The token has two attributes that can match the domain_id.
+    if event.domain_id is not None and event.domain_id not in(
+            token_values['identity_domain_id'],
+            token_values['assignment_domain_id'],):
+        return False
 
-        Compare the values for each level of the tree with the values from
-        the token, accounting for attributes that have alternative
-        keys, and for wildcard matches.
-        if there is a match, continue down the tree.
-        if there is no match, exit early.
+    if event.domain_scope_id is not None and event.domain_scope_id not in (
+            token_values['assignment_domain_id'],):
+        return False
 
-        token_data is a map based on a flattened view of token.
-        The required fields are:
+    # If an event specifies an attribute name, but it does not match, the token
+    # is not revoked.
+    if event.project_id is not None and event.project_id not in (
+            token_values['project_id'],):
+        return False
 
-           'expires_at','user_id', 'project_id', 'identity_domain_id',
-           'assignment_domain_id', 'trust_id', 'trustor_id', 'trustee_id'
-           'consumer_id', 'access_token_id'
+    if event.expires_at is not None and event.expires_at not in (
+            token_values['expires_at'],):
+        return False
 
-        """
-        return self._search(self.revoke_map, _EVENT_NAMES, token_data)
+    if event.trust_id is not None and event.trust_id not in (
+            token_values['trust_id'],):
+        return False
+
+    if event.consumer_id is not None and event.consumer_id not in (
+            token_values['consumer_id'],):
+        return False
+
+    if event.access_token_id is not None and event.access_token_id not in (
+            token_values['access_token_id'],):
+        return False
+
+    if event.audit_id is not None and event.audit_id not in (
+            token_values['audit_id'],):
+        return False
+
+    if event.audit_chain_id is not None and event.audit_chain_id not in (
+            token_values['audit_chain_id'],):
+        return False
+
+    if event.role_id is not None and event.role_id not in (
+            token_values['roles']):
+        return False
+
+    if token_values['issued_at'] > event.issued_before:
+        return False
+
+    return True
 
 
 def build_token_values_v2(access, default_domain_id):
@@ -283,15 +254,15 @@ def build_token_values_v2(access, default_domain_id):
     token_values['identity_domain_id'] = default_domain_id
     token_values['assignment_domain_id'] = default_domain_id
 
-    trust = token_data.get('trust')
+    trust = access.get('trust')
     if trust is None:
         token_values['trust_id'] = None
         token_values['trustor_id'] = None
         token_values['trustee_id'] = None
     else:
         token_values['trust_id'] = trust['id']
-        token_values['trustor_id'] = trust['trustor_id']
-        token_values['trustee_id'] = trust['trustee_id']
+        token_values['trustor_id'] = trust['trustor_user_id']
+        token_values['trustee_id'] = trust['trustee_user_id']
 
     token_values['consumer_id'] = None
     token_values['access_token_id'] = None
@@ -371,3 +342,30 @@ def build_token_values(token_data):
         token_values['consumer_id'] = oauth1['consumer_id']
         token_values['access_token_id'] = oauth1['access_token_id']
     return token_values
+
+
+class _RevokeEventHandler(object):
+    # NOTE(morganfainberg): There needs to be reserved "registry" entries set
+    # in oslo_serialization for application-specific handlers. We picked 127
+    # here since it's waaaaaay far out before oslo_serialization will use it.
+    identity = 127
+    handles = (RevokeEvent,)
+
+    def __init__(self, registry):
+        self._registry = registry
+
+    def serialize(self, obj):
+        return msgpackutils.dumps(obj.__dict__, registry=self._registry)
+
+    def deserialize(self, data):
+        revoke_event_data = msgpackutils.loads(data, registry=self._registry)
+        try:
+            revoke_event = RevokeEvent(**revoke_event_data)
+        except Exception:
+            LOG.debug("Failed to deserialize RevokeEvent. Data is %s",
+                      revoke_event_data)
+            raise
+        return revoke_event
+
+
+cache.register_model_handler(_RevokeEventHandler)
