@@ -44,7 +44,7 @@ class BaseASTChecker(ast.NodeVisitor):
     """
 
     def __init__(self, tree, filename):
-        """This object is created automatically by pep8.
+        """Created object automatically by pep8.
 
         :param tree: an AST tree
         :param filename: name of the file being analyzed
@@ -66,7 +66,7 @@ class BaseASTChecker(ast.NodeVisitor):
 
 
 class CheckForMutableDefaultArgs(BaseASTChecker):
-    """Checks for the use of mutable objects as function/method defaults.
+    """Check for the use of mutable objects as function/method defaults.
 
     We are only checking for list and dict literals at this time. This means
     that a developer could specify an instance of their own and cause a bug.
@@ -115,7 +115,7 @@ def block_comments_begin_with_a_space(physical_line, line_number):
 
 
 class CheckForAssertingNoneEquality(BaseASTChecker):
-    """Ensures that code does not use a None with assert(Not*)Equal."""
+    """Ensure that code does not use a None with assert(Not*)Equal."""
 
     CHECK_DESC_IS = ('K003 Use self.assertIsNone(...) when comparing '
                      'against None')
@@ -193,7 +193,7 @@ class CheckForLoggingIssues(BaseASTChecker):
                 self.visit(value)
 
     def _filter_imports(self, module_name, alias):
-        """Keeps lists of logging and i18n imports."""
+        """Keep lists of logging and i18n imports."""
         if module_name in self.LOG_MODULES:
             self.logger_module_names.append(alias.asname or alias.name)
         elif module_name in self.I18N_MODULES:
@@ -227,7 +227,7 @@ class CheckForLoggingIssues(BaseASTChecker):
             return None
 
     def visit_Assign(self, node):
-        """Look for 'LOG = logging.getLogger'
+        """Look for 'LOG = logging.getLogger'.
 
         This handles the simple case:
           name = [logging_module].getLogger(...)
@@ -395,6 +395,19 @@ class CheckForLoggingIssues(BaseASTChecker):
                 continue
             return []
 
+        def is_in_args(node, name):
+            if (len(node.args) > 0 and isinstance(node.args[0], ast.Name)
+                    and name in (a.id for a in node.args)):
+                return True
+            return False
+
+        def is_in_kwargs(node, name):
+            for keyword in node.keywords:
+                if (isinstance(keyword.value, ast.Name)
+                        and keyword.value.id == name):
+                    return True
+            return False
+
         peers = find_peers(node)
         for peer in peers:
             if isinstance(peer, ast.Raise):
@@ -402,13 +415,13 @@ class CheckForLoggingIssues(BaseASTChecker):
                     exc = peer.exc
                 else:
                     exc = peer.type
-                if (isinstance(exc, ast.Call) and
-                        len(exc.args) > 0 and
-                        isinstance(exc.args[0], ast.Name) and
-                        name in (a.id for a in exc.args)):
-                    return True
-                else:
-                    return False
+                if isinstance(exc, ast.Call):
+                    if is_in_args(exc, name):
+                        return True
+                    elif is_in_kwargs(exc, name):
+                        return True
+
+                return False
             elif isinstance(peer, ast.Assign):
                 if name in (t.id for t in peer.targets if hasattr(t, 'id')):
                     return False
@@ -420,9 +433,9 @@ def dict_constructor_with_sequence_copy(logical_line):
     PEP-0274 introduced dict comprehension with performance enhancement
     and it also makes code more readable.
 
-    Okay: lower_res = {k.lower(): v for k, v in six.iteritems(res[1])}
+    Okay: lower_res = {k.lower(): v for k, v in res[1].items()}
     Okay: fool = dict(a='a', b='b')
-    K008: lower_res = dict((k.lower(), v) for k, v in six.iteritems(res[1]))
+    K008: lower_res = dict((k.lower(), v) for k, v in res[1].items())
     K008:     attrs = dict([(k, _from_json(v))
     K008: dict([[i,i] for i in range(3)])
 
