@@ -12,17 +12,19 @@
 
 import uuid
 
-from oslo_config import cfg
 from oslo_config import fixture as config_fixture
 from oslo_serialization import jsonutils
+import webob
 
 from keystone.auth.plugins import mapped
+import keystone.conf
 from keystone import exception
 from keystone.federation import utils as mapping_utils
 from keystone.tests import unit
 from keystone.tests.unit import mapping_fixtures
 
 
+CONF = keystone.conf.CONF
 FAKE_MAPPING_ID = uuid.uuid4().hex
 
 
@@ -381,7 +383,7 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         self.assertEqual([], mapped_properties['group_ids'])
 
     def test_rule_engine_blacklist_and_direct_groups_mapping_multiples(self):
-        """Tests matching multiple values before the blacklist.
+        """Test matching multiple values before the blacklist.
 
         Verifies that the local indexes are correct when matching multiple
         remote values for a field when the field occurs before the blacklist
@@ -409,7 +411,7 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         self.assertEqual([], mapped_properties['group_ids'])
 
     def test_rule_engine_whitelist_direct_group_mapping_missing_domain(self):
-        """Test if the local rule is rejected upon missing domain value
+        """Test if the local rule is rejected upon missing domain value.
 
         This is a variation with a ``whitelist`` filter.
 
@@ -420,7 +422,7 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         self.assertRaises(exception.ValidationError, rp.process, assertion)
 
     def test_rule_engine_blacklist_direct_group_mapping_missing_domain(self):
-        """Test if the local rule is rejected upon missing domain value
+        """Test if the local rule is rejected upon missing domain value.
 
         This is a variation with a ``blacklist`` filter.
 
@@ -547,15 +549,15 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         as it was not explicitly specified in the mapping.
 
         """
+        request = webob.Request.blank('/')
         mapping = mapping_fixtures.MAPPING_USER_IDS
         rp = mapping_utils.RuleProcessor(FAKE_MAPPING_ID, mapping['rules'])
         assertion = mapping_fixtures.ADMIN_ASSERTION
         mapped_properties = rp.process(assertion)
-        context = {'environment': {}}
         self.assertIsNotNone(mapped_properties)
         self.assertValidMappedUserObject(mapped_properties)
         unique_id, display_name = mapped.get_user_unique_id_and_display_name(
-            context, mapped_properties)
+            request, mapped_properties)
         self.assertEqual('bob', unique_id)
         self.assertEqual('bob', display_name)
 
@@ -568,7 +570,7 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         - Check if the user has proper domain ('federated') set
         - Check if the user has proper type set ('ephemeral')
         - Check if display_name is properly set from the assertion
-        - Check if unique_id is properly set and and equal to value hardcoded
+        - Check if unique_id is properly set and equal to value hardcoded
         in the mapping
 
         This test does two iterations with different assertions used as input
@@ -580,17 +582,17 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         not to change it.
 
         """
+        request = webob.Request.blank('/')
         testcases = [(mapping_fixtures.CUSTOMER_ASSERTION, 'bwilliams'),
                      (mapping_fixtures.EMPLOYEE_ASSERTION, 'tbo')]
         for assertion, exp_user_name in testcases:
             mapping = mapping_fixtures.MAPPING_USER_IDS
             rp = mapping_utils.RuleProcessor(FAKE_MAPPING_ID, mapping['rules'])
             mapped_properties = rp.process(assertion)
-            context = {'environment': {}}
             self.assertIsNotNone(mapped_properties)
             self.assertValidMappedUserObject(mapped_properties)
             unique_id, display_name = (
-                mapped.get_user_unique_id_and_display_name(context,
+                mapped.get_user_unique_id_and_display_name(request,
                                                            mapped_properties)
             )
             self.assertEqual(exp_user_name, display_name)
@@ -699,7 +701,7 @@ class MappingRuleEngineTests(unit.BaseTestCase):
                          mapped_properties['group_names'][0]['name'])
 
     def test_rule_engine_group_ids_mapping_whitelist(self):
-        """Test mapping engine when group_ids is explicitly set
+        """Test mapping engine when group_ids is explicitly set.
 
         Also test whitelists on group ids
 
@@ -759,7 +761,7 @@ class TestUnicodeAssertionData(unit.BaseTestCase):
 
     def setUp(self):
         super(TestUnicodeAssertionData, self).setUp()
-        self.config_fixture = self.useFixture(config_fixture.Config(cfg.CONF))
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
         self.config_fixture.config(group='federation',
                                    assertion_prefix='PFX')
 
@@ -778,8 +780,10 @@ class TestUnicodeAssertionData(unit.BaseTestCase):
         # pulled from the HTTP headers. These bytes may be decodable as
         # ISO-8859-1 according to Section 3.2.4 of RFC 7230. Let's assume
         # that our web server plugins are correctly encoding the data.
-        context = dict(environment=mapping_fixtures.UNICODE_NAME_ASSERTION)
-        data = mapping_utils.get_assertion_params_from_env(context)
+        request = webob.Request.blank(
+            '/path',
+            environ=mapping_fixtures.UNICODE_NAME_ASSERTION)
+        data = mapping_utils.get_assertion_params_from_env(request)
         # NOTE(dstanek): keystone.auth.plugins.mapped
         return dict(data)
 
