@@ -1,4 +1,4 @@
-# Copyright 2012 OpenStack LLC
+# Copyright 2012 OpenStack Foundation
 # Copyright 2013 IBM Corp.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,12 +13,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import time
+import datetime
 
-from oslo_config import cfg
+import freezegun
 from oslo_serialization import jsonutils
 from six.moves import range
 
+import keystone.conf
 from keystone.tests import unit
 from keystone.tests.unit import filtering
 from keystone.tests.unit import ksfixtures
@@ -26,7 +27,7 @@ from keystone.tests.unit.ksfixtures import temporaryfile
 from keystone.tests.unit import test_v3
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 class IdentityTestFilteredCase(filtering.FilterTests,
@@ -97,7 +98,7 @@ class IdentityTestFilteredCase(filtering.FilterTests,
             policyfile.write(jsonutils.dumps(new_policy))
 
     def test_list_users_filtered_by_domain(self):
-        """GET /users?domain_id=mydomain (filtered)
+        """GET /users?domain_id=mydomain (filtered).
 
         Test Plan:
 
@@ -115,7 +116,7 @@ class IdentityTestFilteredCase(filtering.FilterTests,
         self.assertIn(self.user3['id'], id_list)
 
     def test_list_filtered_domains(self):
-        """GET /domains?enabled=0
+        """GET /domains?enabled=0.
 
         Test Plan:
 
@@ -158,7 +159,7 @@ class IdentityTestFilteredCase(filtering.FilterTests,
         self.assertIn(CONF.identity.default_domain_id, id_list)
 
     def test_multiple_filters(self):
-        """GET /domains?enabled&name=myname
+        """GET /domains?enabled&name=myname.
 
         Test Plan:
 
@@ -178,7 +179,7 @@ class IdentityTestFilteredCase(filtering.FilterTests,
         self.assertIs(True, r.result.get('domains')[0]['enabled'])
 
     def test_invalid_filter_is_ignored(self):
-        """GET /domains?enableds&name=myname
+        """GET /domains?enableds&name=myname.
 
         Test Plan:
 
@@ -201,7 +202,7 @@ class IdentityTestFilteredCase(filtering.FilterTests,
         self.assertIs(True, r.result.get('domains')[0]['enabled'])
 
     def test_list_users_filtered_by_funny_name(self):
-        """GET /users?name=%myname%
+        """GET /users?name=%myname%.
 
         Test Plan:
 
@@ -210,23 +211,26 @@ class IdentityTestFilteredCase(filtering.FilterTests,
         - Ensure we can filter on it
 
         """
-        self._set_policy({"identity:list_users": []})
-        user = self.user1
-        user['name'] = '%my%name%'
-        self.identity_api.update_user(user['id'], user)
+        # NOTE(lbragstad): Since Fernet tokens do not support sub-second
+        # precision we must freeze the clock and ensure we increment the time
+        # by a full second after a recovation event has occurred. Otherwise the
+        # token will be considered revoked even though it is actually a valid
+        # token.
+        time = datetime.datetime.utcnow()
+        with freezegun.freeze_time(time) as frozen_datetime:
 
-        # NOTE(breton): the sleep below is required because time
-        # in revocations and token was rounded down. In Newton
-        # release freezegun is used for this purpose instead of
-        # sleep. Freezegun cannot be used in Mitaka release, because
-        # it was not in requirements when release happened.
-        time.sleep(1)
+            self._set_policy({"identity:list_users": []})
+            user = self.user1
+            user['name'] = '%my%name%'
+            self.identity_api.update_user(user['id'], user)
 
-        url_by_name = '/users?name=%my%name%'
-        r = self.get(url_by_name, auth=self.auth)
+            frozen_datetime.tick(delta=datetime.timedelta(seconds=1))
 
-        self.assertEqual(1, len(r.result.get('users')))
-        self.assertEqual(user['id'], r.result.get('users')[0]['id'])
+            url_by_name = '/users?name=%my%name%'
+            r = self.get(url_by_name, auth=self.auth)
+
+            self.assertEqual(1, len(r.result.get('users')))
+            self.assertEqual(user['id'], r.result.get('users')[0]['id'])
 
     def test_inexact_filters(self):
         # Create 20 users
@@ -281,7 +285,7 @@ class IdentityTestFilteredCase(filtering.FilterTests,
         url_by_name = '/users?name__endswith=of'
         r = self.get(url_by_name, auth=self.auth)
         self.assertEqual(1, len(r.result.get('users')))
-        self.assertEqual(r.result.get('users')[0]['id'], user_list[7]['id'])
+        self.assertEqual(user_list[7]['id'], r.result.get('users')[0]['id'])
 
         url_by_name = '/users?name__iendswith=OF'
         r = self.get(url_by_name, auth=self.auth)
@@ -292,7 +296,7 @@ class IdentityTestFilteredCase(filtering.FilterTests,
         self._delete_test_data('user', user_list)
 
     def test_filter_sql_injection_attack(self):
-        """GET /users?name=<injected sql_statement>
+        """GET /users?name=<injected sql_statement>.
 
         Test Plan:
 
@@ -320,7 +324,7 @@ class IdentityTestFilteredCase(filtering.FilterTests,
         # Check group table is still there...
         url_by_name = "/groups"
         r = self.get(url_by_name, auth=self.auth)
-        self.assertTrue(len(r.result.get('groups')) > 0)
+        self.assertGreater(len(r.result.get('groups')), 0)
 
 
 class IdentityTestListLimitCase(IdentityTestFilteredCase):
@@ -371,7 +375,7 @@ class IdentityTestListLimitCase(IdentityTestFilteredCase):
             self.policy_api.delete_policy(policy['id'])
 
     def _test_entity_list_limit(self, entity, driver):
-        """GET /<entities> (limited)
+        """GET /<entities> (limited).
 
         Test Plan:
 
