@@ -15,29 +15,46 @@
 import copy
 import uuid
 
-from oslo_config import cfg
+import mock
+from oslo_log import versionutils
 from oslo_serialization import jsonutils
 from pycadf import cadftaxonomy
 from six.moves import http_client
 from six.moves import urllib
 
-from keystone.contrib import oauth1
-from keystone.contrib.oauth1 import controllers
-from keystone.contrib.oauth1 import core
+import keystone.conf
+from keystone.contrib.oauth1 import routers
 from keystone import exception
+from keystone import oauth1
+from keystone.oauth1.backends import base
+from keystone.oauth1 import controllers
+from keystone.tests import unit
 from keystone.tests.unit.common import test_notifications
+from keystone.tests.unit import ksfixtures
 from keystone.tests.unit.ksfixtures import temporaryfile
 from keystone.tests.unit import test_v3
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
+
+
+def _urllib_parse_qs_text_keys(content):
+    results = urllib.parse.parse_qs(content)
+    return {key.decode('utf-8'): value for key, value in results.items()}
+
+
+class OAuth1ContribTests(test_v3.RestfulTestCase):
+
+    @mock.patch.object(versionutils, 'report_deprecated_feature')
+    def test_exception_happens(self, mock_deprecator):
+        routers.OAuth1Extension(mock.ANY)
+        mock_deprecator.assert_called_once_with(mock.ANY, mock.ANY)
+        args, _kwargs = mock_deprecator.call_args
+        self.assertIn("Remove oauth1_extension from", args[1])
 
 
 class OAuth1Tests(test_v3.RestfulTestCase):
 
-    EXTENSION_NAME = 'oauth1'
-    EXTENSION_TO_ADD = 'oauth1_extension'
-
     CONSUMER_URL = '/OS-OAUTH1/consumers'
 
     def setUp(self):
@@ -93,6 +110,8 @@ class OAuth1Tests(test_v3.RestfulTestCase):
         return endpoint, headers, ref
 
     def _authorize_request_token(self, request_id):
+        if isinstance(request_id, bytes):
+            request_id = request_id.decode()
         return '/OS-OAUTH1/authorize/%s' % (request_id)
 
 
@@ -140,7 +159,7 @@ class ConsumerCRUDTests(OAuth1Tests):
         consumer = self._create_single_consumer()
         consumer_id = consumer['id']
         resp = self.delete(self.CONSUMER_URL + '/%s' % consumer_id)
-        self.assertResponseStatus(resp, 204)
+        self.assertResponseStatus(resp, http_client.NO_CONTENT)
 
     def test_consumer_get(self):
         consumer = self._create_single_consumer()
@@ -253,8 +272,8 @@ class OAuthFlowTests(OAuth1Tests):
                                                   self.project_id)
         content = self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
-        credentials = urllib.parse.parse_qs(content.result)
+            response_content_type='application/x-www-form-urlencoded')
+        credentials = _urllib_parse_qs_text_keys(content.result)
         request_key = credentials['oauth_token'][0]
         request_secret = credentials['oauth_token_secret'][0]
         self.request_token = oauth1.Token(request_key, request_secret)
@@ -262,9 +281,9 @@ class OAuthFlowTests(OAuth1Tests):
 
         url = self._authorize_request_token(request_key)
         body = {'roles': [{'id': self.role_id}]}
-        resp = self.put(url, body=body, expected_status=200)
+        resp = self.put(url, body=body, expected_status=http_client.OK)
         self.verifier = resp.result['token']['oauth_verifier']
-        self.assertTrue(all(i in core.VERIFIER_CHARS for i in self.verifier))
+        self.assertTrue(all(i in base.VERIFIER_CHARS for i in self.verifier))
         self.assertEqual(8, len(self.verifier))
 
         self.request_token.set_verifier(self.verifier)
@@ -272,8 +291,8 @@ class OAuthFlowTests(OAuth1Tests):
                                                  self.request_token)
         content = self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
-        credentials = urllib.parse.parse_qs(content.result)
+            response_content_type='application/x-www-form-urlencoded')
+        credentials = _urllib_parse_qs_text_keys(content.result)
         access_key = credentials['oauth_token'][0]
         access_secret = credentials['oauth_token_secret'][0]
         self.access_token = oauth1.Token(access_key, access_secret)
@@ -303,13 +322,15 @@ class AccessTokenCRUDTests(OAuthFlowTests):
 
     def test_get_single_access_token(self):
         self.test_oauth_flow()
+        access_token_key_string = self.access_token.key.decode()
+
         url = '/users/%(user_id)s/OS-OAUTH1/access_tokens/%(key)s' % {
               'user_id': self.user_id,
-              'key': self.access_token.key
+              'key': access_token_key_string
         }
         resp = self.get(url)
         entity = resp.result['access_token']
-        self.assertEqual(self.access_token.key, entity['id'])
+        self.assertEqual(access_token_key_string, entity['id'])
         self.assertEqual(self.consumer['key'], entity['consumer_id'])
         self.assertEqual('http://localhost/v3' + url, entity['links']['self'])
 
@@ -323,15 +344,17 @@ class AccessTokenCRUDTests(OAuthFlowTests):
         self.test_oauth_flow()
         resp = self.get('/users/%(id)s/OS-OAUTH1/access_tokens/%(key)s/roles'
                         % {'id': self.user_id,
-                           'key': self.access_token.key})
+                           'key': self.access_token.key.decode()})
         entities = resp.result['roles']
         self.assertTrue(entities)
         self.assertValidListLinks(resp.result['links'])
 
     def test_get_role_in_access_token(self):
         self.test_oauth_flow()
+
+        access_token_key = self.access_token.key.decode()
         url = ('/users/%(id)s/OS-OAUTH1/access_tokens/%(key)s/roles/%(role)s'
-               % {'id': self.user_id, 'key': self.access_token.key,
+               % {'id': self.user_id, 'key': access_token_key,
                   'role': self.role_id})
         resp = self.get(url)
         entity = resp.result['role']
@@ -339,8 +362,10 @@ class AccessTokenCRUDTests(OAuthFlowTests):
 
     def test_get_role_in_access_token_dne(self):
         self.test_oauth_flow()
+
+        access_token_key = self.access_token.key.decode()
         url = ('/users/%(id)s/OS-OAUTH1/access_tokens/%(key)s/roles/%(role)s'
-               % {'id': self.user_id, 'key': self.access_token.key,
+               % {'id': self.user_id, 'key': access_token_key,
                   'role': uuid.uuid4().hex})
         self.get(url, expected_status=http_client.NOT_FOUND)
 
@@ -353,11 +378,12 @@ class AccessTokenCRUDTests(OAuthFlowTests):
         self.assertTrue(entities)
         self.assertValidListLinks(resp.result['links'])
 
+        access_token_key = self.access_token.key.decode()
         # Delete access_token
         resp = self.delete('/users/%(user)s/OS-OAUTH1/access_tokens/%(auth)s'
                            % {'user': self.user_id,
-                              'auth': self.access_token.key})
-        self.assertResponseStatus(resp, 204)
+                              'auth': access_token_key})
+        self.assertResponseStatus(resp, http_client.NO_CONTENT)
 
         # List access_token should be 0
         resp = self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens'
@@ -367,7 +393,7 @@ class AccessTokenCRUDTests(OAuthFlowTests):
         self.assertValidListLinks(resp.result['links'])
 
 
-class AuthTokenTests(OAuthFlowTests):
+class AuthTokenTests(object):
 
     def test_keystone_token_is_valid(self):
         self.test_oauth_flow()
@@ -378,7 +404,7 @@ class AuthTokenTests(OAuthFlowTests):
 
         # now verify the oauth section
         oauth_section = r.result['token']['OS-OAUTH1']
-        self.assertEqual(self.access_token.key,
+        self.assertEqual(self.access_token.key.decode(),
                          oauth_section['access_token_id'])
         self.assertEqual(self.consumer['key'], oauth_section['consumer_id'])
 
@@ -388,7 +414,7 @@ class AuthTokenTests(OAuthFlowTests):
         self.assertEqual(self.role_id, roles_list[0]['id'])
 
         # verify that the token can perform delegated tasks
-        ref = self.new_user_ref(domain_id=self.domain_id)
+        ref = unit.new_user_ref(domain_id=self.domain_id)
         r = self.admin_request(path='/v3/users', headers=headers,
                                method='POST', body={'user': ref})
         self.assertValidUserResponse(r, ref)
@@ -396,11 +422,12 @@ class AuthTokenTests(OAuthFlowTests):
     def test_delete_access_token_also_revokes_token(self):
         self.test_oauth_flow()
 
+        access_token_key = self.access_token.key.decode()
         # Delete access token
         resp = self.delete('/users/%(user)s/OS-OAUTH1/access_tokens/%(auth)s'
                            % {'user': self.user_id,
-                              'auth': self.access_token.key})
-        self.assertResponseStatus(resp, 204)
+                              'auth': access_token_key})
+        self.assertResponseStatus(resp, http_client.NO_CONTENT)
 
         # Check Keystone Token no longer exists
         headers = {'X-Subject-Token': self.keystone_token_id,
@@ -415,7 +442,7 @@ class AuthTokenTests(OAuthFlowTests):
         consumer_id = self.consumer['key']
         resp = self.delete('/OS-OAUTH1/consumers/%(consumer_id)s'
                            % {'consumer_id': consumer_id})
-        self.assertResponseStatus(resp, 204)
+        self.assertResponseStatus(resp, http_client.NO_CONTENT)
 
         # List access_token should be 0
         resp = self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens'
@@ -491,7 +518,7 @@ class AuthTokenTests(OAuthFlowTests):
                           self.keystone_token_id)
 
     def _create_trust_get_token(self):
-        ref = self.new_trust_ref(
+        ref = unit.new_trust_ref(
             trustor_user_id=self.user_id,
             trustee_user_id=self.user_id,
             project_id=self.project_id,
@@ -521,8 +548,8 @@ class AuthTokenTests(OAuthFlowTests):
                                                   self.project_id)
         content = self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
-        credentials = urllib.parse.parse_qs(content.result)
+            response_content_type='application/x-www-form-urlencoded')
+        credentials = _urllib_parse_qs_text_keys(content.result)
         request_key = credentials['oauth_token'][0]
         request_secret = credentials['oauth_token_secret'][0]
         self.request_token = oauth1.Token(request_key, request_secret)
@@ -534,7 +561,7 @@ class AuthTokenTests(OAuthFlowTests):
 
     def test_oauth_token_cannot_create_new_trust(self):
         self.test_oauth_flow()
-        ref = self.new_trust_ref(
+        ref = unit.new_trust_ref(
             trustor_user_id=self.user_id,
             trustee_user_id=self.user_id,
             project_id=self.project_id,
@@ -588,6 +615,30 @@ class AuthTokenTests(OAuthFlowTests):
                  expected_status=http_client.FORBIDDEN)
 
 
+class FernetAuthTokenTests(AuthTokenTests, OAuthFlowTests):
+
+    def config_overrides(self):
+        super(FernetAuthTokenTests, self).config_overrides()
+        self.config_fixture.config(group='token', provider='fernet')
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
+
+    def test_delete_keystone_tokens_by_consumer_id(self):
+        self.skipTest('Fernet tokens are never persisted in the backend.')
+
+
+class UUIDAuthTokenTests(AuthTokenTests, OAuthFlowTests):
+
+    def config_overrides(self):
+        super(UUIDAuthTokenTests, self).config_overrides()
+        self.config_fixture.config(group='token', provider='uuid')
+
+
 class MaliciousOAuth1Tests(OAuth1Tests):
 
     def test_bad_consumer_secret(self):
@@ -606,7 +657,7 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         url, headers = self._create_request_token(consumer, self.project_id)
         self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
+            response_content_type='application/x-www-form-urlencoded')
         url = self._authorize_request_token(uuid.uuid4().hex)
         body = {'roles': [{'id': self.role_id}]}
         self.put(url, body=body, expected_status=http_client.NOT_FOUND)
@@ -637,15 +688,15 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         url, headers = self._create_request_token(consumer, self.project_id)
         content = self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
-        credentials = urllib.parse.parse_qs(content.result)
+            response_content_type='application/x-www-form-urlencoded')
+        credentials = _urllib_parse_qs_text_keys(content.result)
         request_key = credentials['oauth_token'][0]
         request_secret = credentials['oauth_token_secret'][0]
         request_token = oauth1.Token(request_key, request_secret)
 
         url = self._authorize_request_token(request_key)
         body = {'roles': [{'id': self.role_id}]}
-        resp = self.put(url, body=body, expected_status=200)
+        resp = self.put(url, body=body, expected_status=http_client.OK)
         verifier = resp.result['token']['oauth_verifier']
         self.assertIsNotNone(verifier)
 
@@ -663,8 +714,8 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         url, headers = self._create_request_token(consumer, self.project_id)
         content = self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
-        credentials = urllib.parse.parse_qs(content.result)
+            response_content_type='application/x-www-form-urlencoded')
+        credentials = _urllib_parse_qs_text_keys(content.result)
         request_key = credentials['oauth_token'][0]
 
         self.assignment_api.remove_role_from_user_and_project(
@@ -687,8 +738,8 @@ class MaliciousOAuth1Tests(OAuth1Tests):
                                                   self.project_id)
         content = self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
-        credentials = urllib.parse.parse_qs(content.result)
+            response_content_type='application/x-www-form-urlencoded')
+        credentials = _urllib_parse_qs_text_keys(content.result)
         request_key = credentials['oauth_token'][0]
         request_secret = credentials['oauth_token_secret'][0]
         self.request_token = oauth1.Token(request_key, request_secret)
@@ -710,8 +761,8 @@ class MaliciousOAuth1Tests(OAuth1Tests):
                                                   self.project_id)
         content = self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
-        credentials = urllib.parse.parse_qs(content.result)
+            response_content_type='application/x-www-form-urlencoded')
+        credentials = _urllib_parse_qs_text_keys(content.result)
         request_key = credentials['oauth_token'][0]
         request_secret = credentials['oauth_token_secret'][0]
         self.request_token = oauth1.Token(request_key, request_secret)
@@ -719,7 +770,7 @@ class MaliciousOAuth1Tests(OAuth1Tests):
 
         url = self._authorize_request_token(request_key)
         body = {'roles': [{'id': self.role_id}]}
-        resp = self.put(url, body=body, expected_status=200)
+        resp = self.put(url, body=body, expected_status=http_client.OK)
         self.verifier = resp.result['token']['oauth_verifier']
 
         self.request_token.set_verifier(self.verifier)
@@ -727,8 +778,8 @@ class MaliciousOAuth1Tests(OAuth1Tests):
                                                  self.request_token)
         content = self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
-        credentials = urllib.parse.parse_qs(content.result)
+            response_content_type='application/x-www-form-urlencoded')
+        credentials = _urllib_parse_qs_text_keys(content.result)
         access_key = credentials['oauth_token'][0]
         access_secret = credentials['oauth_token_secret'][0]
         self.access_token = oauth1.Token(access_key, access_secret)
@@ -753,7 +804,8 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         # NOTE(stevemar): To simulate this error, we remove the Authorization
         # header from the post request.
         del headers['Authorization']
-        self.post(endpoint, headers=headers, expected_status=500)
+        self.post(endpoint, headers=headers,
+                  expected_status=http_client.INTERNAL_SERVER_ERROR)
 
 
 class OAuthNotificationTests(OAuth1Tests,
@@ -793,14 +845,13 @@ class OAuthNotificationTests(OAuth1Tests,
                                 cadftaxonomy.SECURITY_ACCOUNT)
 
     def test_oauth_flow_notifications(self):
-        """Test to ensure notifications are sent for oauth tokens
+        """Test to ensure notifications are sent for oauth tokens.
 
         This test is very similar to test_oauth_flow, however
         there are additional checks in this test for ensuring that
         notifications for request token creation, and access token
         creation/deletion are emitted.
         """
-
         consumer = self._create_single_consumer()
         consumer_id = consumer['id']
         consumer_secret = consumer['secret']
@@ -811,27 +862,28 @@ class OAuthNotificationTests(OAuth1Tests,
                                                   self.project_id)
         content = self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
-        credentials = urllib.parse.parse_qs(content.result)
+            response_content_type='application/x-www-form-urlencoded')
+        credentials = _urllib_parse_qs_text_keys(content.result)
         request_key = credentials['oauth_token'][0]
         request_secret = credentials['oauth_token_secret'][0]
         self.request_token = oauth1.Token(request_key, request_secret)
         self.assertIsNotNone(self.request_token.key)
 
+        request_key_string = request_key.decode()
         # Test to ensure the create request token notification is sent
-        self._assert_notify_sent(request_key,
+        self._assert_notify_sent(request_key_string,
                                  test_notifications.CREATED_OPERATION,
                                  'OS-OAUTH1:request_token')
-        self._assert_last_audit(request_key,
+        self._assert_last_audit(request_key_string,
                                 test_notifications.CREATED_OPERATION,
                                 'OS-OAUTH1:request_token',
                                 cadftaxonomy.SECURITY_CREDENTIAL)
 
         url = self._authorize_request_token(request_key)
         body = {'roles': [{'id': self.role_id}]}
-        resp = self.put(url, body=body, expected_status=200)
+        resp = self.put(url, body=body, expected_status=http_client.OK)
         self.verifier = resp.result['token']['oauth_verifier']
-        self.assertTrue(all(i in core.VERIFIER_CHARS for i in self.verifier))
+        self.assertTrue(all(i in base.VERIFIER_CHARS for i in self.verifier))
         self.assertEqual(8, len(self.verifier))
 
         self.request_token.set_verifier(self.verifier)
@@ -839,32 +891,33 @@ class OAuthNotificationTests(OAuth1Tests,
                                                  self.request_token)
         content = self.post(
             url, headers=headers,
-            response_content_type='application/x-www-urlformencoded')
-        credentials = urllib.parse.parse_qs(content.result)
+            response_content_type='application/x-www-form-urlencoded')
+        credentials = _urllib_parse_qs_text_keys(content.result)
         access_key = credentials['oauth_token'][0]
         access_secret = credentials['oauth_token_secret'][0]
         self.access_token = oauth1.Token(access_key, access_secret)
         self.assertIsNotNone(self.access_token.key)
 
+        access_key_string = access_key.decode()
         # Test to ensure the create access token notification is sent
-        self._assert_notify_sent(access_key,
+        self._assert_notify_sent(access_key_string,
                                  test_notifications.CREATED_OPERATION,
                                  'OS-OAUTH1:access_token')
-        self._assert_last_audit(access_key,
+        self._assert_last_audit(access_key_string,
                                 test_notifications.CREATED_OPERATION,
                                 'OS-OAUTH1:access_token',
                                 cadftaxonomy.SECURITY_CREDENTIAL)
 
         resp = self.delete('/users/%(user)s/OS-OAUTH1/access_tokens/%(auth)s'
                            % {'user': self.user_id,
-                              'auth': self.access_token.key})
-        self.assertResponseStatus(resp, 204)
+                              'auth': self.access_token.key.decode()})
+        self.assertResponseStatus(resp, http_client.NO_CONTENT)
 
         # Test to ensure the delete access token notification is sent
-        self._assert_notify_sent(access_key,
+        self._assert_notify_sent(access_key_string,
                                  test_notifications.DELETED_OPERATION,
                                  'OS-OAUTH1:access_token')
-        self._assert_last_audit(access_key,
+        self._assert_last_audit(access_key_string,
                                 test_notifications.DELETED_OPERATION,
                                 'OS-OAUTH1:access_token',
                                 cadftaxonomy.SECURITY_CREDENTIAL)
@@ -873,7 +926,7 @@ class OAuthNotificationTests(OAuth1Tests,
 class OAuthCADFNotificationTests(OAuthNotificationTests):
 
     def setUp(self):
-        """Repeat the tests for CADF notifications """
+        """Repeat the tests for CADF notifications."""
         super(OAuthCADFNotificationTests, self).setUp()
         self.config_fixture.config(notification_format='cadf')
 
