@@ -13,10 +13,10 @@
 import copy
 import uuid
 
-from oslo_config import cfg
 from oslo_utils import timeutils
 from six.moves import range
 
+import keystone.conf
 from keystone import exception
 from keystone.federation import constants as federation_constants
 from keystone.models import token_model
@@ -24,7 +24,7 @@ from keystone.tests.unit import core
 from keystone.tests.unit import test_token_provider
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 class TestKeystoneTokenModel(core.TestCase):
@@ -60,6 +60,8 @@ class TestKeystoneTokenModel(core.TestCase):
         self.assertEqual(
             self.v3_sample_token['token']['project']['domain']['name'],
             token_data.project_domain_name)
+        self.assertEqual(
+            self.v3_sample_token['token']['is_domain'], token_data.is_domain)
         self.assertEqual(self.v3_sample_token['token']['OS-TRUST:trust']['id'],
                          token_data.trust_id)
         self.assertEqual(
